@@ -176,9 +176,6 @@ static char *get_extended_arelt_filename PARAMS ((bfd *arch,
 static boolean do_slurp_bsd_armap PARAMS ((bfd *abfd));
 static boolean do_slurp_coff_armap PARAMS ((bfd *abfd));
 static const char *normalize PARAMS ((const char *file));
-static boolean bfd_construct_extended_name_table PARAMS ((bfd *abfd,
-							  char **tabloc,
-							  unsigned int *));
 static struct areltdata *bfd_ar_hdr_from_filesystem PARAMS ((bfd *abfd,
 							     const char *));
 static boolean compute_and_write_armap PARAMS ((bfd *arch,
@@ -1155,6 +1152,32 @@ normalize (file)
 }
 #endif
 
+/* Build a BFD style extended name table.  */
+
+boolean
+_bfd_archive_bsd_construct_extended_name_table (abfd, tabloc, tablen, name)
+     bfd *abfd;
+     char **tabloc;
+     bfd_size_type *tablen;
+     const char **name;
+{
+  *name = "ARFILENAMES/";
+  return _bfd_construct_extended_name_table (abfd, false, tabloc, tablen);
+}
+
+/* Build an SVR4 style extended name table.  */
+
+boolean
+_bfd_archive_coff_construct_extended_name_table (abfd, tabloc, tablen, name)
+     bfd *abfd;
+     char **tabloc;
+     bfd_size_type *tablen;
+     const char **name;
+{
+  *name = "//";
+  return _bfd_construct_extended_name_table (abfd, true, tabloc, tablen);
+}
+
 /* Follows archive_head and produces an extended name table if
    necessary.  Returns (in tabloc) a pointer to an extended name
    table, and in tablen the length of the table.  If it makes an entry
@@ -1163,14 +1186,14 @@ normalize (file)
    something went wrong.  A successful return may still involve a
    zero-length tablen!  */
 
-static boolean
-bfd_construct_extended_name_table (abfd, tabloc, tablen)
+boolean
+_bfd_construct_extended_name_table (abfd, trailing_slash, tabloc, tablen)
      bfd *abfd;
+     boolean trailing_slash;
      char **tabloc;
-     unsigned int *tablen;
+     bfd_size_type *tablen;
 {
   unsigned int maxname = abfd->xvec->ar_max_namelen;
-  boolean elf_style = ar_padchar (abfd) == '/';
   unsigned int total_namelen = 0;
   bfd *current;
   char *strptr;
@@ -1193,7 +1216,7 @@ bfd_construct_extended_name_table (abfd, tabloc, tablen)
 	{
 	  /* Add one to leave room for \n.  */
 	  total_namelen += thislen + 1;
-	  if (elf_style)
+	  if (trailing_slash)
 	    {
 	      /* Leave room for trailing slash.  */
 	      ++total_namelen;
@@ -1233,7 +1256,7 @@ bfd_construct_extended_name_table (abfd, tabloc, tablen)
 	     generalise this hack. */
 	  struct ar_hdr *hdr = arch_hdr (current);
 	  strcpy (strptr, normal);
-	  if (! elf_style)
+	  if (! trailing_slash)
 	    strptr[thislen] = '\012';
 	  else
 	    {
@@ -1253,7 +1276,7 @@ bfd_construct_extended_name_table (abfd, tabloc, tablen)
 		*temp = ' ';
 	  }
 	  strptr += thislen + 1;
-	  if (elf_style)
+	  if (trailing_slash)
 	    ++strptr;
 	}
     }
@@ -1476,7 +1499,8 @@ _bfd_write_archive_contents (arch)
 {
   bfd *current;
   char *etable = NULL;
-  unsigned int elength = 0;
+  bfd_size_type elength = 0;
+  const char *ename = NULL;
   boolean makemap = bfd_has_map (arch);
   boolean hasobjects = false;	/* if no .o's, don't bother to make a map */
   bfd_size_type wrote;
@@ -1517,7 +1541,8 @@ _bfd_write_archive_contents (arch)
 	}
     }
 
-  if (!bfd_construct_extended_name_table (arch, &etable, &elength))
+  if (!BFD_SEND (arch, _bfd_construct_extended_name_table,
+		 (arch, &etable, &elength, &ename)))
     return false;
 
   if (bfd_seek (arch, (file_ptr) 0, SEEK_SET) != 0)
@@ -1541,10 +1566,7 @@ _bfd_write_archive_contents (arch)
       struct ar_hdr hdr;
 
       memset ((char *) (&hdr), 0, sizeof (struct ar_hdr));
-      if (ar_padchar (arch) == '/')
-	sprintf (&(hdr.ar_name[0]), "//");
-      else
-	sprintf (&(hdr.ar_name[0]), "ARFILENAMES/");
+      strcpy (hdr.ar_name, ename);
       sprintf (&(hdr.ar_size[0]), "%-10d", (int) elength);
       strncpy (hdr.ar_fmag, ARFMAG, 2);
       for (i = 0; i < sizeof (struct ar_hdr); i++)
