@@ -1,5 +1,5 @@
 /* BFD back-end for Hitachi H8/300 COFF binaries.
-   Copyright 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+   Copyright 1990, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
    Written by Steve Chamberlain, <sac@cygnus.com>.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -20,13 +20,14 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
-#include "libbfd.h"
 #include "obstack.h"
+#include "libbfd.h"
+#include "bfdlink.h"
 #include "coff/h8300.h"
 #include "coff/internal.h"
-#include "seclet.h"
 #include "libcoff.h"
-extern bfd_error_vector_type bfd_error_vector;
+
+#define COFF_DEFAULT_SECTION_ALIGNMENT_POWER (1)
 
 static reloc_howto_type howto_table[] =
 {
@@ -42,7 +43,7 @@ static reloc_howto_type howto_table[] =
   HOWTO (R_JMP2, 0, 0, 8, false, 0, complain_overflow_bitfield, 0, "pcrecl/16", true, 0x000000ff, 0x000000ff, false),
 
 
-  HOWTO (R_JMPL1, 0, 1, 32, false, 0, complain_overflow_bitfield, 0, "24/pcrell", true, 0x0000ffff, 0x0000ffff, false),
+  HOWTO (R_JMPL1, 0, 2, 32, false, 0, complain_overflow_bitfield, 0, "24/pcrell", true, 0x00ffffff, 0x00ffffff, false),
   HOWTO (R_JMPL_B8, 0, 0, 8, false, 0, complain_overflow_bitfield, 0, "pc8/24", true, 0x000000ff, 0x000000ff, false),
 
   HOWTO (R_MOVLB1, 0, 1, 16, false, 0, complain_overflow_bitfield, 0, "24/8", true, 0x0000ffff, 0x0000ffff, false),
@@ -54,7 +55,7 @@ static reloc_howto_type howto_table[] =
 /* Turn a howto into a reloc number */
 
 #define SELECT_RELOC(x,howto) \
-  { x = select_reloc(howto); }
+  { x.r_type = select_reloc(howto); }
 
 #define BADMAG(x) (H8300BADMAG(x)&& H8300HBADMAG(x))
 #define H8300 1			/* Customize coffcode.h */
@@ -149,12 +150,12 @@ rtype2howto (internal, dst)
  reloc_processing(relent, reloc, symbols, abfd, section)
 
 static void
-DEFUN (reloc_processing, (relent, reloc, symbols, abfd, section),
-       arelent * relent AND
-       struct internal_reloc *reloc AND
-       asymbol ** symbols AND
-       bfd * abfd AND
-       asection * section)
+reloc_processing (relent, reloc, symbols, abfd, section)
+     arelent * relent;
+     struct internal_reloc *reloc;
+     asymbol ** symbols;
+     bfd * abfd;
+     asection * section;
 {
   relent->address = reloc->r_vaddr;
   rtype2howto (relent, reloc);
@@ -165,7 +166,7 @@ DEFUN (reloc_processing, (relent, reloc, symbols, abfd, section),
     }
   else
     {
-      relent->sym_ptr_ptr = &(bfd_abs_symbol);
+      relent->sym_ptr_ptr = bfd_abs_section_ptr->symbol_ptr_ptr;
     }
 
 
@@ -178,11 +179,12 @@ DEFUN (reloc_processing, (relent, reloc, symbols, abfd, section),
 
 
 static int
-h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
+h8300_reloc16_estimate(abfd, input_section, reloc, shrink, link_info)
+     bfd *abfd;
      asection *input_section;
-     asymbol **symbols;
      arelent *reloc;
      unsigned int shrink;
+     struct bfd_link_info *link_info;
 {
   bfd_vma value;  
   bfd_vma dot;
@@ -207,8 +209,8 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
 
       /* Thing is a move one byte */
     case R_MOVB1:
-      value = bfd_coff_reloc16_get_value(reloc,0);
-	
+      value = bfd_coff_reloc16_get_value(reloc, link_info, input_section);
+
       if (value >= 0xff00)
 	{ 
 
@@ -218,7 +220,7 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
 	  /* The place to relc moves back by one */
 	  /* This will be two bytes smaller in the long run */
 	  shrink +=2 ;
-	  bfd_perform_slip(symbols, 2, input_section, address);
+	  bfd_perform_slip(abfd, 2, input_section, address);
 	}      
 
       break;
@@ -227,7 +229,7 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
        actual data */
 
     case R_JMPL1:
-      value = bfd_coff_reloc16_get_value(reloc, 0);
+      value = bfd_coff_reloc16_get_value(reloc, link_info, input_section);
 	
       dot = input_section->output_section->vma +
 	input_section->output_offset + address;
@@ -246,13 +248,13 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
 	  reloc->howto = reloc->howto + 1;	  
 	  /* This will be two bytes smaller in the long run */
 	  shrink +=2 ;
-	  bfd_perform_slip(symbols, 2, input_section, address);
+	  bfd_perform_slip(abfd, 2, input_section, address);
 	}
       break;
 
     case R_JMP1:
 
-      value = bfd_coff_reloc16_get_value(reloc, 0);
+      value = bfd_coff_reloc16_get_value(reloc, link_info, input_section);
 	
       dot = input_section->output_section->vma +
 	input_section->output_offset + address;
@@ -274,7 +276,7 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
 
 	  /* This will be two bytes smaller in the long run */
 	  shrink +=2 ;
-	  bfd_perform_slip(symbols, 2, input_section, address);
+	  bfd_perform_slip(abfd, 2, input_section, address);
 	}
       break;
     }
@@ -296,9 +298,11 @@ h8300_reloc16_estimate(input_section, symbols, reloc, shrink)
 */
 
 static void
-h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
+h8300_reloc16_extra_cases (abfd, link_info, link_order, reloc, data, src_ptr,
+			   dst_ptr)
      bfd *abfd;
-     struct bfd_seclet *seclet;
+     struct bfd_link_info *link_info;
+     struct bfd_link_order *link_order;
      arelent *reloc;
      bfd_byte *data;
      unsigned int *src_ptr;
@@ -306,6 +310,7 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
 {
   unsigned int src_address = *src_ptr;
   unsigned int dst_address = *dst_ptr;
+  asection *input_section = link_order->u.indirect.section;
 
   switch (reloc->howto->type)
     {
@@ -313,35 +318,73 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
 	 the byte before the 24bit hole, so we can treat it as a 32bit pointer */
     case R_PCRBYTE:
       {
-	bfd_vma dot = seclet->offset 
+	bfd_vma dot = link_order->offset 
 	  + dst_address 
-	    + seclet->u.indirect.section->output_section->vma;
-	int gap = bfd_coff_reloc16_get_value (reloc, seclet) - dot;
+	    + link_order->u.indirect.section->output_section->vma;
+	int gap = (bfd_coff_reloc16_get_value (reloc, link_info, input_section)
+		   - dot);
 	if (gap > 127 || gap < -128)
 	  {
-	    bfd_error_vector.reloc_value_truncated (reloc, seclet);
+	    if (! ((*link_info->callbacks->reloc_overflow)
+		   (link_info, bfd_asymbol_name (*reloc->sym_ptr_ptr),
+		    reloc->howto->name, reloc->addend, input_section->owner,
+		    input_section, reloc->address)))
+	      abort ();
 	  }
 
 	bfd_put_8 (abfd, gap, data + dst_address);
 	dst_address++;
 	src_address++;
 
+	break;
+      }
+    case R_PCRWORD:
+      {
+	bfd_vma dot = link_order->offset 
+	  + dst_address 
+	    + link_order->u.indirect.section->output_section->vma;
+	int gap = (bfd_coff_reloc16_get_value (reloc, link_info, input_section)
+		   - dot) - 1;
+	if (gap > 32767 || gap < -32768)
+	  {
+	    if (! ((*link_info->callbacks->reloc_overflow)
+		   (link_info, bfd_asymbol_name (*reloc->sym_ptr_ptr),
+		    reloc->howto->name, reloc->addend, input_section->owner,
+		    input_section, reloc->address)))
+	      abort ();
+	  }
+
+	bfd_put_16 (abfd, gap, data + dst_address);
+	dst_address+=2;
+	src_address+=2;
+
 	break;
       }
 
     case R_RELBYTE:
       {
-	unsigned int gap = bfd_coff_reloc16_get_value (reloc, seclet);
-	if (gap > 0xff && gap < ~0xff)
+	unsigned int gap = bfd_coff_reloc16_get_value (reloc, link_info,
+						       input_section);
+	if (gap < 0xff 
+	    || (gap >= 0x0000ff00
+	        && gap <= 0x0000ffff)
+  	    || (   gap >= 0x00ffff00 
+		&& gap <= 0x00ffffff)
+	    || (   gap >= 0xffffff00
+		&& gap <= 0xffffffff))
 	  {
-	    bfd_error_vector.reloc_value_truncated (reloc, seclet);
+	    bfd_put_8 (abfd, gap, data + dst_address);
+	    dst_address += 1;
+	    src_address += 1;
+	  }
+	else
+	  {
+	    if (! ((*link_info->callbacks->reloc_overflow)
+		   (link_info, bfd_asymbol_name (*reloc->sym_ptr_ptr),
+		    reloc->howto->name, reloc->addend, input_section->owner,
+		    input_section, reloc->address)))
+	      abort ();
 	  }
-
-	bfd_put_8 (abfd, gap, data + dst_address);
-	dst_address += 1;
-	src_address += 1;
-
-
       }
       break;
     case R_JMP1:
@@ -350,13 +393,15 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
       /* A relword which would like to have been modified but
 	     didn't make it */
     case R_RELWORD:
-      bfd_put_16 (abfd, bfd_coff_reloc16_get_value (reloc, seclet),
+      bfd_put_16 (abfd,
+		  bfd_coff_reloc16_get_value (reloc, link_info, input_section),
 		  data + dst_address);
       dst_address += 2;
       src_address += 2;
       break;
     case R_RELLONG:
-      bfd_put_32 (abfd, bfd_coff_reloc16_get_value (reloc, seclet),
+      bfd_put_32 (abfd,
+		  bfd_coff_reloc16_get_value (reloc, link_info, input_section),
 		  data + dst_address);
       dst_address += 4;
       src_address += 4;
@@ -389,7 +434,8 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
       /* the offset must fit ! after all, what was all the relaxing
 	     about ? */
 
-      bfd_put_8 (abfd, bfd_coff_reloc16_get_value (reloc, seclet),
+      bfd_put_8 (abfd,
+		 bfd_coff_reloc16_get_value (reloc, link_info, input_section),
 		 data + dst_address);
 
       /* Note the magic - src goes up by two bytes, but dst by only
@@ -403,11 +449,12 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
       
       /* Speciial relaxed type */
       {
-	bfd_vma dot = seclet->offset
+	bfd_vma dot = link_order->offset
 	+ dst_address
-	+ seclet->u.indirect.section->output_section->vma;
+	+ link_order->u.indirect.section->output_section->vma;
 
-	int gap = bfd_coff_reloc16_get_value (reloc, seclet) - dot - 1;
+	int gap = (bfd_coff_reloc16_get_value (reloc, link_info, input_section)
+		   - dot - 1);
 
 	if ((gap & ~0xff) != 0 && ((gap & 0xff00) != 0xff00))
 	  abort ();
@@ -439,11 +486,12 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
       
       /* Speciial relaxed type */
       {
-	bfd_vma dot = seclet->offset
+	bfd_vma dot = link_order->offset
 	+ dst_address
-	+ seclet->u.indirect.section->output_section->vma;
+	+ link_order->u.indirect.section->output_section->vma;
 
-	int gap = bfd_coff_reloc16_get_value (reloc, seclet) - dot - 2;
+	int gap = (bfd_coff_reloc16_get_value (reloc, link_info, input_section)
+		   - dot - 2);
 
 	if ((gap & ~0xff) != 0 && ((gap & 0xff00) != 0xff00))
 	  abort ();
@@ -473,7 +521,7 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
 
     case R_JMPL1:
       {
-	int v = bfd_coff_reloc16_get_value (reloc, seclet);
+	int v = bfd_coff_reloc16_get_value (reloc, link_info, input_section);
 	int o = bfd_get_32 (abfd, data + src_address);
 	v = (v & 0x00ffffff) | (o & 0xff000000);
 	bfd_put_32 (abfd, v, data + dst_address);
@@ -488,7 +536,7 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
 	 the byte before the 24bit hole, so we can treat it as a 32bit pointer */
     case R_MOVLB1:
       {
-	int v = bfd_coff_reloc16_get_value (reloc, seclet);
+	int v = bfd_coff_reloc16_get_value (reloc, link_info, input_section);
 	int o = bfd_get_32 (abfd, data + dst_address);
 	v = (v & 0x00ffffff) | (o & 0xff000000);
 	bfd_put_32 (abfd, v, data + dst_address);
@@ -516,12 +564,13 @@ h8300_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
 
 #undef coff_bfd_get_relocated_section_contents
 #undef coff_bfd_relax_section
-#define  coff_bfd_get_relocated_section_contents bfd_coff_reloc16_get_relocated_section_contents
+#define coff_bfd_get_relocated_section_contents \
+  bfd_coff_reloc16_get_relocated_section_contents
 #define coff_bfd_relax_section bfd_coff_reloc16_relax_section
 
 
 
-bfd_target h8300coff_vec =
+const bfd_target h8300coff_vec =
 {
   "coff-h8300",			/* name */
   bfd_target_coff_flavour,
@@ -550,6 +599,15 @@ bfd_target h8300coff_vec =
   {bfd_false, coff_write_object_contents,	/* bfd_write_contents */
    _bfd_write_archive_contents, bfd_false},
 
-  JUMP_TABLE (coff),
+     BFD_JUMP_TABLE_GENERIC (coff),
+     BFD_JUMP_TABLE_COPY (coff),
+     BFD_JUMP_TABLE_CORE (_bfd_nocore),
+     BFD_JUMP_TABLE_ARCHIVE (_bfd_archive_coff),
+     BFD_JUMP_TABLE_SYMBOLS (coff),
+     BFD_JUMP_TABLE_RELOCS (coff),
+     BFD_JUMP_TABLE_WRITE (coff),
+     BFD_JUMP_TABLE_LINK (coff),
+     BFD_JUMP_TABLE_DYNAMIC (_bfd_nodynamic),
+
   COFF_SWAP_TABLE,
 };
